"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const graphile_build_pg_1 = require("graphile-build-pg");
/**
 * @description "Set unique constraint using smart comments"
 * @param builder
 */
exports.uniqueConstraintFromSmartComment = (builder) => {
    builder.hook('build', (build) => {
        const { pgIntrospectionResultsByKind: { class: classes, constraint }, } = build;
        classes.forEach((c) => {
            // Only allow fake unique constraints for views
            // for mat view and table, it should be set by query. not smart comment
            const uniqueTag = c.tags.unique;
            if (uniqueTag && c.classKind === 'v') {
                // figure out the attributes
                if (typeof uniqueTag === 'string') {
                    const text = uniqueTag;
                    const attNames = text.substring(text.indexOf('(') + 1, text.indexOf(')'));
                    const attriNums = [];
                    const attrs = [];
                    attNames.split(',').forEach((n) => {
                        const ind = c.attributes.findIndex(a => a.name === n);
                        const attr = c.attributes[ind];
                        if (!attr) {
                            throw new Error(`Cannot find attribute ${n} for unique constraint ${uniqueTag}`);
                        }
                        attriNums.push(ind);
                        attrs.push(attr);
                    });
                    const id = `${Math.random()}`;
                    const uniqueConstraint = {
                        id,
                        classId: c.id,
                        description: 'Unique Constraint',
                        tags: {},
                        class: c,
                        comment: '',
                        foreignClass: undefined,
                        foreignClassId: undefined,
                        foreignKeyAttributeNums: [],
                        foreignKeyAttributes: [],
                        keyAttributeNums: attriNums,
                        keyAttributes: attrs,
                        kind: graphile_build_pg_1.PgEntityKind.CONSTRAINT,
                        name: `${id}_unique_constraint`,
                        type: 'u',
                        namespace: c.namespace,
                        isIndexed: true,
                    };
                    constraint.push(uniqueConstraint);
                    c.constraints.push(uniqueConstraint);
                }
            }
        });
        return build;
    });
};
//# sourceMappingURL=unique_constraint_smart_comment.js.map